#!/usr/bin/sh
#
#	Installation script for Customer Specific Manual C122.02
#	Kitted as part of Kit TCV5117-C0012202-11406-Manual-20010814
#	On 2001-08-14  11:25:05
# 
# @DEC_COPYRIGHT@
#
# HISTORY
# $Log: manual_install_template.sh,v $
# $EndLog$
# 
# @(#)$RCSfile: manual_install_template.sh,v $ $Revision: 1.1.2.3 $ (DEC) $Date: 2001/06/07 08:49:55 $
# 
MANUAL_PATCH_TOOLS_REV=001

# Define a clean PATH
PATH=/sbin:/usr/sbin:/usr/bin:/usr/ccs/bin
export PATH

#
#	Un-documented, Un-supported - use root other than /
#
DEFAULT_ROOT=/
[ "$_ROOT" ] || {
	_ROOT=$DEFAULT_ROOT
}
_SMDB=$_ROOT/usr/.smdb.

#
#	Setup manual patch adm area if it doesn't exist
#
MANUAL_PATCH_ROOT=`echo $_ROOT/var/adm/manual_patch | sed 's[//[/['`
MANUAL_PATCH_BACKUP=$MANUAL_PATCH_ROOT/backup
MANUAL_PATCH_DOC=$MANUAL_PATCH_ROOT/doc
MANUAL_PATCH_LOGS=$MANUAL_PATCH_ROOT/log
MANUAL_PATCH_DB=$MANUAL_PATCH_ROOT/db
MANUAL_PATCH_BIN=$MANUAL_PATCH_ROOT/bin
[ ! -d "$MANUAL_PATCH_ROOT" ] && {
	mkdir -p $MANUAL_PATCH_ROOT
	mkdir -p $MANUAL_PATCH_BACKUP
	mkdir -p $MANUAL_PATCH_DOC/doc
	mkdir -p $MANUAL_PATCH_LOGS
	mkdir -p $MANUAL_PATCH_DB
}


MANUAL_PATCH_INSTALL_ENTRY_LOG=$MANUAL_PATCH_LOGS/Install.log

[ ! -e "$MANUAL_PATCH_INSTALL_ENTRY_LOG" ] && {
	touch $MANUAL_PATCH_INSTALL_ENTRY_LOG
}
#
#	Per-patch definitions
#
MANUAL_PATCH_KIT_NAME="TCV5117-C0012202-11406-Manual-20010814"
MANUAL_PATCH_DESCRIPTION="Patch: Network superpatch"
MANUAL_PATCH_CATEGORY="Cluster Kernel Patches"
MANUAL_PATCH_NEW_PATCH_ID="C122.02"
MANUAL_PATCH_FULL_PATCH_ID="TruCluster Server V5.1 Patch C122.02"
MANUAL_PATCH_SUBSET_NAME="TCRPATC0012202510"
MANUAL_PATCH_ORIGINAL_SUBSETS="TCRBASE510"
MANUAL_PATCH_KERNEL_REBUILD="1"

#
#	Are we in the correct context ?
#
: CheckExecutionContext
CheckExecutionContext(){
	[ "`whoami`" != "root" ] && {
		echo "Script must be executed as root"
		return 1
	}
	return 0
}
#
#	Get a timestamp
#
: GetTimeStamp
GetTimeStamp(){
	date +"%Y-%m-%d-%H:%M:%S"
}
#
#	Log installation
#
: LogInstallation
LogInstallation(){
	[ "$#" != "1" ] && {
		return 1
	}
	_SubsetName=$1
	shift

	# Do we want to skip logging
	[ "$_MANUAL_PATCH_SKIP_LOGGING" ] && return 0

	# Is logging already in progress ?
	[ "$MANUAL_PATCH_LOGFILE" != "" ] && {
		return 0
	}
	MANUAL_PATCH_LOGFILE=$MANUAL_PATCH_LOGS/$_SubsetName
	export MANUAL_PATCH_LOGFILE
	exec log $MANUAL_PATCH_LOGFILE $0

	# Should never get here
	return 1
}
#
#	Check for instructions left lying around
# Has anyone left around instructions for us ?
# (Just like the SCP extension mechanism).
#
MANUAL_PATCH_SCPEXT=$MANUAL_PATCH_BIN/ManualPatchSCPEXT
CheckForInstructions(){
	[ -e "$MANUAL_PATCH_SCPEXT" ] || return 0

	[ -x "$MANUAL_PATCH_SCPEXT" ] && {
		$MANUAL_PATCH_SCPEXT "$0" $* || {
			exit 3
		}
	}
}
#
#	Convert file name space correctly
#
ChangeFileNameSpace(){
	FileName=$1

	FileName=`echo $FileName | sed -e 's/\/\.upd\.\./\//' -e 's/\/\.new\.\./\//`
	echo $FileName
}
#
#	Save files now being laid down
#
SaveExistingFiles() {
	RootDirectory=$1
	shift 
	FileList=$*

	( cd $RootDirectory

		[ "$PATCH_DEBUG" ] && echo "Now saving existing files"
	
		for File in $FileList
		do
			SystemNameSpaceFile=`ChangeFileNameSpace $File`
			[ ! -e "$SystemNameSpaceFile" ] && {
				echo "File $SystemNameSpaceFile isn't on the system - probably since it is a new file"
				[ "$PATCH_DEBUG" ] && echo "Can't find file $SystemNameSpaceFile"
				continue
			}
			SavedFileName=$SystemNameSpaceFile.PreManualCSP.$TimeStamp
			[ -e "$SavedFileName" ] && {
				echo "Already have saved file name $SavedFileName"
				exit 3
			}
			ln $SystemNameSpaceFile $SavedFileName
			[ "$PATCH_DEBUG" ] && echo "Saved $File to $SavedFileName"
			# TBD Save the .proto.. and the .new.. files ?
		done
		return 0
	) || return 1
	return 0
}
#
#	Lay files down
#
InstallFiles(){
	RootDirectory=$1
	shift
	TarFileName=$1
	shift

	cat $TarFileName | ( cd $RootDirectory ; tar xfp -) || return 1
	return 0
}
#
#	Some files were not laid down in the correct name space, rectify this now.
#
MoveFilesToNameSpace(){
	RootDirectory=$1
	shift
	FileList=$*
	

	(cd $RootDirectory
		HadError=0
		for FileName in $FileList
		do
			SystemNameSpaceFileName=`ChangeFileNameSpace $FileName`
			[ "$SystemNameSpaceFileName" = "$FileName" ] && {
				[ "$PATCH_DEBUG" ] && echo "No name space change for $FileName"
				continue
			}
			# TBD, handle .new.. files.
			(echo $FileName | fgrep -q /.new..) || {
				[ "$PATCH_DEBUG" ] && echo "Moving $FileName into $SystemNameSpaceFileName"
				mv $FileName $SystemNameSpaceFileName || HadError=1
				continue
			}
			[ "$PATCH_DEBUG" ] && echo "Merging file $FileName"
			echo "Don't forget to run the merge script for $SystemNameSpaceFileName"
		done
		return $HadError
	) || return 1
	return 0
}
#
#	Copy over the documentation
#
CopyOverDocumentation(){
	PatchId=$1
	shift
	InputDirectory=$1
	shift
	OutputDirectory=$1
	shift

	HadError=0
	DOCUMENTATION_SUFFIXES="abs txt special_instructions problem_ids rcs_strings"

	[ "$PATCH_DEBUG" ] && echo "Copying over documentation"

	for Suffix in $DOCUMENTATION_SUFFIXES
	do
		Document=$InputDirectory/$PatchId.$Suffix

		[ ! -e "$Document" ] && {
			# Special instructions are optional
			[ "$Suffix" = "special_instructions" ] && {
				continue
			}
			echo "Can't find document $Document"
			HadError=1
			continue
		}
		cp -p $Document $OutputDirectory/
	done

	ReleaseNotes=$InputDirectory/$MANUAL_PATCH_KIT_NAME.txt
	if [ ! -e "$ReleaseNotes" ]; then
		echo "Can't find release notes $ReleaseNotes"
		HadError=1
	else
		cp -p $ReleaseNotes $OutputDirectory/
	fi
	return $HadError
}
#
#	Output Header of execution
#
OutputHeader() {
	echo 
	echo "Now installing $MANUAL_PATCH_FULL_PATCH_ID at $TimeStamp (Logging to $MANUAL_PATCH_LOGFILE)"
	echo
	return 0
}
#
#	Output finishing message
#
OutputFooter() {
	echo
	echo "Finished installing $MANUAL_PATCH_FULL_PATCH_ID at `GetTimeStamp`"
	echo
	[ "$MANUAL_PATCH_KERNEL_REBUILD" ] && {
		echo "Please note that you need to re-build the kernel and reboot"
		echo "so that this patch will take effect"
	}
	return 0
}
#
#	Check whether original subsets are installed
#
CheckForOriginalSubsets() {
	_OriginalSubsetList=$*

	HadError=0

	for _OriginalSubsetName in $_OriginalSubsetList
	do
		# Check if subset isn't installed.
		[ -e $_SMDB/$_OriginalSubsetName.lk ] || {
			_DESC="Subset $_OriginalSubsetName"
			[ -e $_SMDB/$_OriginalSubsetName.ctrl ] && {
				_FULLDESC=`grep \^DESC= $_SMDB/$_OriginalSubsetName.ctrl | 
					sed "s/DESC=.//" | cut -d% -f1`
				_DESC="$_FULLDESC ($_DESC)"
			}
			# Print out header if this is the first one found missing
			[ "$HadError" = "0" ] && {
				echo "This patch contains files originally shipped in the following subset(s)"
				echo
			}
			echo "$_DESC"
			HadError=1
		}
	done
	[ "$HadError" = "0" ] || {
		echo
		echo "However, these subset(s) are not installed on your system, therefore this patch"
		echo "will not be installed"
	}

	return $HadError
}
#
#	Log Installation
#
#	Copy .sh to ./db
#	Create .FileList in ./db
#
#	Create entry in Install.log in the following format:
#	INSTALL%TimeStamp%KitName%FullPatchId%PatchId%Description%Category
LogInstallationEvent(){

	# Put things in $MANUAL_PATCH_DB.
	cp $0 $MANUAL_PATCH_DB
	echo $MANUAL_PATCH_FILE_LIST > $MANUAL_PATCH_DB/$MANUAL_PATCH_SUBSET_NAME.FileList
	touch $MANUAL_PATCH_DB/$MANUAL_PATCH_SUBSET_NAME.Installed


	#
	# Create entry in installation log file
	#

	_KeyWord="INSTALL"
	_TimeStamp="$TimeStamp"
	_KitName="$MANUAL_PATCH_KIT_NAME"
	_FullPatchId="$MANUAL_PATCH_FULL_PATCH_ID"
	_Description="$MANUAL_PATCH_DESCRIPTION"
	_Category="$MANUAL_PATCH_CATEGORY"

	echo "$_KeyWord%$_TimeStamp%$_KitName%$_FullPatchId%$_Description%$_Category" >> $MANUAL_PATCH_INSTALL_ENTRY_LOG
}
#
#	Is this manual patch installed already ?
#
IsPatchInstalled(){
	_SubsetName=$1

	[ -e "$MANUAL_PATCH_DB/$_SubsetName.Installed" ] && {
		# Allow an override
		[ "$_MANUAL_PATCH_OVERRIDE" != "" ] && {
			echo "Allowing re-installation of patch"
			return 1
		}
		return 0
	}
	return 1
}
#
#	Is anything of this product code installed ?
#
ValidateProductVersion(){
	_ProductCode=$1
	_ProductVersionCode=$2

	_INSTALLED=`(cd $_SMDB; ls -1 $_ProductCode*$_ProductVersionCode.lk 2>/dev/null | sed 's/\.lk//g' | sed 's/.*[\?\*].*//g') | head -10`
	[ -z "$_INSTALLED" ] && return 1
	return 0
}
Main() {
	[ "$#" != "0" ] && {
		echo "$0: No options allowed"
		exit 3
	}
	MANUAL_PATCH_ID=`basename $0 | sed 's/.install.sh//'`
	LogInstallation $MANUAL_PATCH_ID || {
		echo "Can't log installation"
		exit 1
	}

	OutputHeader || exit 3


	CheckExecutionContext || exit 3
	# Has anyone left around instructions for us ?
	# (Just like the SCP extension mechanism).
	CheckForInstructions || exit 3

	TimeStamp=`GetTimeStamp`
	KIT_LOCATION=`dirname $0`

	[ "$MANUAL_PATCH_ID" != "$MANUAL_PATCH_SUBSET_NAME" ] && {
		echo "The base name of the file $MANUAL_PATCH_ID has changed from it's original \
			($MANUAL_PATCH_SUBSET_NAME)"
		exit 3
	}

	# Parse subset name
	MANUAL_PATCH_PRODUCT_CODE=`expr "$MANUAL_PATCH_SUBSET_NAME" : '^\(...\)'`
	MANUAL_PATCH_IS_CSP=`expr "$MANUAL_PATCH_SUBSET_NAME" : '^...PAT\(.\)'`
	MANUAL_PATCH_MAJOR_VERSION=`expr "$MANUAL_PATCH_SUBSET_NAME" : '^...PAT.\(.....\)'`
	MANUAL_PATCH_MINOR_VERSION=`expr "$MANUAL_PATCH_SUBSET_NAME" : '^...PAT......\(..\)'`
	MANUAL_PATCH_PRODUCT_VERSION_CODE=`expr "$MANUAL_PATCH_SUBSET_NAME" : '^...PAT........\(...\)'`

	ValidateProductVersion $MANUAL_PATCH_PRODUCT_CODE $MANUAL_PATCH_PRODUCT_VERSION_CODE || {
		echo "No subsets of this product ($MANUAL_PATCH_PRODUCT_CODE $MANUAL_PATCH_PRODUCT_VERSION_CODE) are installed on this system"
		exit 3
	}
	IsPatchInstalled $MANUAL_PATCH_SUBSET_NAME && {
		echo "Patch is already installed"
		exit 3
	}
	
	CheckForOriginalSubsets $MANUAL_PATCH_ORIGINAL_SUBSETS || exit 3

	TAR_FILE_NAME=$KIT_LOCATION/$MANUAL_PATCH_SUBSET_NAME.tar
	[ ! -e "$TAR_FILE_NAME" ] && {
		echo "Can't find tar file $TAR_FILE_NAME"
		exit 3
	}
	MANUAL_PATCH_FILE_LIST=`tar tf $TAR_FILE_NAME | cut -d " " -f 1`
	SaveExistingFiles $_ROOT $MANUAL_PATCH_FILE_LIST || exit 3
	InstallFiles $_ROOT $TAR_FILE_NAME || exit 3
	MoveFilesToNameSpace $_ROOT $MANUAL_PATCH_FILE_LIST || exit 3
	CopyOverDocumentation $MANUAL_PATCH_SUBSET_NAME $KIT_LOCATION/../Documentation/txt $MANUAL_PATCH_DOC || exit 3
	LogInstallationEvent || exit 3
	OutputFooter || exit 3
	return 0
}
Main $*
